#ifndef CALIBRATIONCONTROLLERIMPL_H
#define CALIBRATIONCONTROLLERIMPL_H

#include "ipc/CalibrationController.h"
#include "IS/CalibrationControllerStatus.h"
#include <boost/noncopyable.hpp>
#include <boost/thread.hpp>
#include <boost/shared_ptr.hpp>

#include <memory>

using std::auto_ptr;

namespace SctCalibrationController {

class Test;
class ConfigUpdater;
class RunControl;
class SctApiException;
using namespace Ipc;

/**
   Implimentation of the CalibrationController interface.
   A singleton, it controls the synchronisation of the various tasks required 
   during testing and calibration. 
   
   There is an abort method which allows the premature termination of a sequence or test after the end of the current scan.

   Maintains a reference to a SctApi, to which it forwards requests for scans.
   Requests for sequencees, tests and scans can be made only when in control.

   Publishes it's status in IS as CalibrationControllerStatus.
*/
    
class CalibrationControllerImpl : public CalibrationController, public boost::noncopyable {
public:
    static CalibrationControllerImpl& initialize(RunControl& rc);
    static CalibrationControllerImpl& instance();    
        
    //Overrides
    virtual ScanLibrary& getScanLibrary() const;
    virtual TestLibrary& getTestLibrary() const;
    virtual SequenceLibrary& getSequenceLibrary() const;
    virtual void doScan (Sct_SctApi_T_Scan* s);
    virtual void doTest (boost::shared_ptr<TestRequest> t);
    virtual void doSequence (boost::shared_ptr<SequenceRequest> r);
    virtual void abort();
    virtual Sct_SctApi_T_Scan* getScan(unsigned long runNumber, unsigned long scanNumber) const;    
    virtual void setUpdateOption( Sct_CalibrationController_T_CalibrationController_UpdateOption );

    //Internal for communication with RunControl:
    void setApi(Sct_SctApi_T_SctApi& api);
    void takeControl(unsigned int runNumber);
    void giveupControl();
    void reset();

    //Internal for use by other classes
    void executeSequence(SequenceRequest& sr);
    auto_ptr<Test> executeTest(TestRequest& tr);
    void executeScan (Sct_SctApi_T_Scan* s, unsigned int index);
    Sct_SctApi_T_SctApi* getApi() const;
    unsigned int getRunNumber() const;
    unsigned int getNextScanNumber() const;
    
    void applyChanges(const unsigned long runNumber, const unsigned long scanNumber, const std::list<std::string>& list );

    //static void applyChangesCallback(ISCallbackInfo * isc);

private:    
    CalibrationControllerImpl(RunControl& rc);
    virtual ~CalibrationControllerImpl();    
    void updateStatus();
    
    bool isInControl();
    bool isBusy();
    void setBusy(bool busy);
    Sct_SctApi_T_SctApi* api;
    
    void sctApiError(SctApiException& e);
    void setNextScanNumber(unsigned int scanNumber);
    
    /**
       Utility function: get all modules in all modulegroups for a particular scan.
    */
    static std::list<string> getAllModulesForScan(Sct_SctApi_T_Scan* scan);

    CalibrationControllerStatus status;
    RunControl& rc;
    unsigned int runNumber;
    unsigned int nextScanNumber;
    bool abortNow;
    static CalibrationControllerImpl* inst;

    boost::thread_group m_sequence_request_thread_group;
    
    friend class SequenceRequestWorker;
};    
}

#endif //#ifndef CALIBRATIONCONTROLLERIMPL_H
