#ifndef MODULEELEMENT_H
#define MODULEELEMENT_H
#include "Sct/Exception.h"
#include "Sct/SctParameters.h"
#include "Sct/OutOfRangeError.h"

using namespace Sct;

namespace SctData {
    /**
     * Represents an contiguous element of a module : may be a chip, link or channel.
     * @author Alan Barr
     * @date 3 July 2003
     */
    class ModuleElement{
    public:
	/** Constructor takes the first and last channel of the element.
	 *   0,127 for chip 0 link 0, or perhaps 768, 895 for chip 0 link 1.
	 * @param first channel contained @param last channel contained
	 * @throw OutOfRangeError if last<first or last>1535.
	 */
	ModuleElement(const unsigned first, const unsigned last);
	/// Destructor
	~ModuleElement() throw() {;}
	/// First channel contained (0->1535)
	unsigned getFirst() const throw() {return first;}
	/// Last channel contained (0->1535)
	unsigned getLast() const throw() {return last;}
	/// @return the number of channels (=last-first+1)
	unsigned getNChannels() const throw();
	/// @return true if only contains one element
	bool isChannel() const throw();
	/// @return true if the channels contained precisely match one chip
	bool isChip() const throw();
	/// @return true if the channels contained precisely match one link
	bool isLink() const throw();
	/// @return true if all of the channels are contained
	bool isModule() const throw();
	/// explicit equality operator @return true if first and last are equal
	bool operator == (const ModuleElement&) const throw();
	/// @return true if this element is contained within or equal to the argument
	bool isWithin(const ModuleElement&) const throw();
	/// @return true if this element contains or is equal to the RHS element
	bool contains(const ModuleElement&) const throw();
	/// @return true if this element overlaps any part of RHS element
	bool overlaps(const ModuleElement&) const throw();
    private:
	unsigned first, last;
	ModuleElement() throw(); ///<can't do this
    };

    //--------------------------------------------------------------------
    /// Represents a chip
    class Chip : public ModuleElement{
    public:
	///@param ichip in the range 0->11
	Chip(const unsigned ichip) throw (LogicError);
	~Chip() throw(){;}
    };
    
    //--------------------------------------------------------------------
    /// Represents a Link
    class Link : public ModuleElement{
    public:
	/// @param ilink in the range 0->1
	Link(const unsigned ilink) throw (LogicError);
	~Link() throw(){;}
    };

    //--------------------------------------------------------------------
    /// Represents a Channel
    class Channel : public ModuleElement{
    public:
	///@param ichannel in the range 0->1535
	Channel(const unsigned ichannel) throw (LogicError);
	~Channel() throw(){;}
    };
     //--------------------------------------------------------------------
    /// Represents a whole Module
    class Module : public ModuleElement{
    public:
	Module() throw (LogicError);
	~Module() throw(){;}
    };
//==========================================================================
// inlines
//==========================================================================
    inline ModuleElement::ModuleElement(const unsigned lo, const unsigned hi) {
#ifndef NDEBUG
	if (hi>nChannelModule ) {
	    throw OutOfRangeError<unsigned>("ModuleElement last", __FILE__, __LINE__, hi, 0, nChannelModule);
	}
	if (lo>hi ){
	    throw OutOfRangeError<unsigned>("ModuleElement first ", __FILE__, __LINE__, lo, 0, hi);
	}
#endif
	last=hi;
	first=lo;

    }
    inline unsigned int ModuleElement::getNChannels() const throw() {
	return getLast() - getFirst() + 1;
    }
    inline bool ModuleElement::isChannel() const throw(){
	return (getNChannels() == 1);
    }
    inline bool ModuleElement::isChip() const throw(){
	return (getFirst() % nChannelChip == 0 && getNChannels() == nChannelChip);
    }
    inline bool ModuleElement::isLink() const throw(){
	return (getFirst() % nChannelLink == 0 && getNChannels() == nChannelLink);
    }
     inline bool ModuleElement::isModule() const throw(){
	return (getFirst() % nChannelModule == 0 && getNChannels() == nChannelModule);
    }

    inline bool ModuleElement::operator == (const ModuleElement& el) const throw(){
	return (getFirst()==el.getFirst() && getLast()==el.getLast() );
    }

    inline bool ModuleElement::isWithin(const ModuleElement& el) const throw(){
	return (getFirst()>=el.getFirst() && getLast()<=el.getLast() );
    }
    inline bool ModuleElement::contains(const ModuleElement& el) const throw(){
	return (getFirst()<=el.getFirst() && getLast()>=el.getLast() );
    }
    
    inline bool ModuleElement::overlaps(const ModuleElement& el) const throw(){
	return ( (getFirst()<=el.getLast() && getFirst()>=el.getFirst() )  // my first element contained
		 || ( getLast()<=el.getLast() && getLast()>=el.getFirst() ) ); // my last element contained
    }
    inline Chip::Chip(const unsigned ichip) throw (LogicError) : 
	ModuleElement(ichip*nChannelChip, (ichip+1)*nChannelChip-1) {
    }
    
    inline Link::Link(const unsigned ilink) throw (LogicError) : 
	ModuleElement(ilink*nChannelLink, (ilink+1)*nChannelLink-1) {
    }
    inline Channel::Channel(const unsigned ichannel) throw (LogicError) : 
	ModuleElement(ichannel, ichannel){
    }
    
    inline Module::Module() throw (LogicError) : 
	ModuleElement(0, nChannelModule-1) {
    }

} // end of namespace SctData
#endif //#ifndef MODULEELEMENT_H
