#include "Comp.h"
#include "Sct/IS/IOManagerIS.h"
#include "Sct/Exception.h"
#include "Sct/LogicErrors.h"
#include "Sct/SctNames.h"
#include "Sct/SctParameters.h"
#include "SctData/FitScanResult.h"
#include "SctData/FitObject.h"
#include <is/isinfo.h>
#include <boost/shared_ptr.hpp>
#include <iostream>
#include <vector>

using namespace Sct;
using namespace Sct::IS;
using namespace boost;
using namespace SctData;
using namespace std;

bool debug = false;
double meanTol = 1;
double sigmaTol = 0.31;
string lastScanName = "FittedData.SctData::FitScanResult.551.53.SCTTestAPI_PseudoModule";


vector<shared_ptr<FitScanResult> > readAll() {
    vector<shared_ptr<FitScanResult> > fits;
        
    ISInfoIterator it( SctNames::getPartition(), "FittedData", ".*" );
    while (it()) {
	shared_ptr<Serializable> ob = IOManagerIS::instance().read(it);
	shared_ptr<FitScanResult> result = dynamic_pointer_cast<FitScanResult>(ob);
	if (!result) 
	    throw IllegalStateError(string("Failed to read fit: ") + it.name(), __FILE__, __LINE__);
	fits.push_back(result);
    }
    
    return fits;
}

void removeAll() {
    ISInfoIterator it( SctNames::getPartition(), "FittedData", ".*" );
    ISInfoDictionary dict(SctNames::getPartition());
    while (it()) {
	dict.remove(it.name());
    }
}

bool compareFits(FitScanResult& nagFit, FitScanResult& rootFit) {
    Comp cMean(meanTol);
    Comp cSigma(sigmaTol);
    int meanFailCount = 0;
    int sigmaFailCount = 0;
    
    cout << endl << endl << "Comparing " << nagFit.getUniqueID() << endl;
    
    for (unsigned int link = 0; link<nLinkModule; ++link) {
	for (unsigned int i=0; i<nChannelLink; ++i) {
	    FitObject& nagFitOb = nagFit.getChannelFit(link, i);
	    FitObject& rootFitOb = rootFit.getChannelFit(link, i);
	    int nagMeanIndex = nagFitOb.getParIndex("Mean");
	    int nagSigmaIndex = nagFitOb.getParIndex("Sigma");
	    int rootMeanIndex = nagFitOb.getParIndex("Mean");
	    int rootSigmaIndex = nagFitOb.getParIndex("Sigma");
	    
	    if (cMean.comp(nagFitOb.getParameter(nagMeanIndex), rootFitOb.getParameter(rootMeanIndex))) {
		cout << "Channel: " << (link*nChannelLink+i) << " Mean different. Root: " << rootFitOb.getParameter(rootMeanIndex) << "+-" 
		     << rootFitOb.getParError(rootMeanIndex) << " Nag: " << nagFitOb.getParameter(nagMeanIndex) << " +- " 
		     << nagFitOb.getParError(nagMeanIndex) << endl; 
		++meanFailCount;
	    } else if (debug) {
		cout << "Channel: " << (link*nChannelLink+i) << " Mean same. Root: " << rootFitOb.getParameter(rootMeanIndex) << "+-" 
		     << rootFitOb.getParError(rootMeanIndex) << " Nag: " << nagFitOb.getParameter(nagMeanIndex) << " +- " 
		     << nagFitOb.getParError(nagMeanIndex) << endl;
	    }
	    
	    if (cSigma.comp(nagFitOb.getParameter(nagSigmaIndex), rootFitOb.getParameter(rootSigmaIndex))) {
		cout << "Channel: " << (link*nChannelLink+i) << " Sigma different. Root: " << rootFitOb.getParameter(rootSigmaIndex) << "+-" 
		     << rootFitOb.getParError(rootSigmaIndex) << " Nag: " << nagFitOb.getParameter(nagSigmaIndex) << " +- " 
		     << nagFitOb.getParError(nagSigmaIndex) << endl;
		++sigmaFailCount;
	    } else if (debug) {
		cout << "Channel: " << (link*nChannelLink+i) << " Sigma same Root: " << rootFitOb.getParameter(rootSigmaIndex) << "+-" 
		     << rootFitOb.getParError(rootSigmaIndex) << " Nag: " << nagFitOb.getParameter(nagSigmaIndex) << " +- " 
		     << nagFitOb.getParError(nagSigmaIndex) << endl;
	    }
	    
	}
    }
    cout << "Summary.  " << endl << "Av Mean diff: " << cMean.mean() << " +- " << cMean.stddev() << endl;
    cout << "Av Sigma diff: " << cSigma.mean() << " +- " << cSigma.stddev() << endl;
    cout << meanFailCount << " channels failed mean test" << endl;
    cout << sigmaFailCount << " channels failed sigma test" << endl;
    return (meanFailCount + sigmaFailCount < 80);
}


bool compare(vector<shared_ptr<FitScanResult> > nag, vector<shared_ptr<FitScanResult> > root) {    
    if (nag.size() != root.size()) {
	cout << "Numbers of fits different!!  Nag: " << nag.size() << " Root: " << root.size() << endl;
	return false;
    }
    bool retVal = true;
    for (unsigned int i=0; i<nag.size(); ++i) {
	if (!compareFits(*nag[i], *root[i])) retVal = false;
    }
    return retVal;
}


void waitFor(string name, int nsecs) {
    ISInfoDictionary d(SctNames::getPartition());
    int tries = 0;
    while (!d.contains(name.c_str()) && tries < nsecs) {
	if (!sleep(1))
	    ++tries;
    }
    if (tries == nsecs) {
	throw IllegalStateError("Didn't wait long enough", __FILE__, __LINE__);
    }
    if (debug) {
	cout << "Waited for " << tries << " secs" << endl;
    }
}


int main(int argc, char** argv) {
    Sct::setExceptionHandlers(argv[0]);

    try {
	waitFor(lastScanName, 20);
	vector<shared_ptr<FitScanResult> > nag = readAll();
	removeAll();
	system("java -Dipc.ref.file=$IPC_REF_FILE bsh.Interpreter SetFitStrategyRoot.bsh");
	waitFor(lastScanName, 40);
	vector<shared_ptr<FitScanResult> > root = readAll();
	
	bool retVal = compare(nag, root);
	system("java -Dipc.ref.file=$IPC_REF_FILE bsh.Interpreter FittingServiceStatus.bsh");
	if (retVal) {
	    if (debug) cout << "Success!" << endl;
	    return 0;
	}
	else {
	    cout << "Failed" << endl;
	    return 1;
	}
	
    } catch (Throwable& e) {
	e.sendToMrs(MRS_FATAL);
	return 1;
    }
}
