/*
 * ColorScale.java
 *
 * Created on 26 June 2003, 14:34
 */

package DisplayGUI;
import java.awt.Color;
import java.util.Vector;
import java.awt.Graphics2D.*;
import java.awt.geom.*;
import java.awt.GradientPaint;
import java.awt.FontMetrics;
/**
 * Class to hold and draw color scale information
 * @author  robinson
 */
public class ColorScale implements displayParams {
    Vector values;                                         ///<Vector of physical values in scale
    Vector colors;                                         ///<Vector of colors in scale
    Vector labels;                                         ///<Vector of axis labels in scale
    double firstDataPoint;                                 ///<Value of first data point in scale
    double lastDataPoint;                                  ///<Value of last data point in scale
    boolean isGradientPaint;                               ///<Does scale contain gradient colors?
    boolean logScale;                                      ///<Display log scale

    /** Creates a new instance of colorScale with gradient colors */
    public ColorScale(double xlow, Color loColor, String loLabel, double xHigh, Color hiColor, String hiLabel) {
        values = new Vector();
        colors = new Vector();
        labels = new Vector();
        values.addElement(new Double(xlow));
        values.addElement(new Double(xHigh));
        firstDataPoint=xlow;
        lastDataPoint=xHigh;
        colors.addElement(loColor);
        colors.addElement(hiColor);
        labels.addElement(loLabel);
        labels.addElement(hiLabel);
        isGradientPaint=true;
        logScale=false;
    }

    /**  Creates an instance of colorScale with solid colors */
    public ColorScale(double xlow, Color loColor, String loLabel) {
        values = new Vector();
        colors = new Vector();
        labels = new Vector();
        values.addElement(new Double(xlow));
        firstDataPoint=xlow;
        lastDataPoint=xlow;
        colors.addElement(loColor);
        labels.addElement(loLabel);
        isGradientPaint=false;
        logScale=false;
        }
    public void setLogScale() {
        logScale=true;
        }

    /**  Add a new point on the scale with specified value, color, and label */
    public void add(double value, Color color, String label) {
        boolean dataInserted=false;
        for(int i=0;i<values.size();i++) {
            double theValue = ((Double)(values.elementAt(i))).doubleValue();
            if(value==theValue) return; // tried to add same point twice
            if(value<theValue) {
                values.insertElementAt(new Double(value),i);
                colors.insertElementAt(color,i);
                labels.insertElementAt(label,i);
                if(value<firstDataPoint) firstDataPoint=value;
                dataInserted=true;
                break;
            }
        }
        if(!dataInserted) {
            values.addElement(new Double(value));
            colors.addElement(color);
            labels.addElement(label);
            if(value>lastDataPoint) lastDataPoint=value;
        }
        
    }

    /**  Draw the color scale at x,y */    
    public void drawColorScale(java.awt.Graphics2D g2, int x, int y) {

            if(!isGradientPaint) {
                 drawSolidColorScale(g2,x,y);
                 return;
                 }
            if(logScale) {
                lastDataPoint = Math.log(lastDataPoint)/Math.log(10.0);
                firstDataPoint = Math.log(firstDataPoint)/Math.log(10.0);
                }
//            System.out.println("drawColorScale:");
//            for(int i=0;i<values.size();i++) System.out.println("values["+i+"]="+((Double)values.elementAt(i)).toString());
//            for(int i=0;i<colors.size();i++) System.out.println("colors["+i+"]="+((Color)colors.elementAt(i)).toString());
//            for(int i=0;i<labels.size();i++) System.out.println("labels["+i+"]="+(String)labels.elementAt(i));
            Rectangle2D.Double drawRect;
            double lastValue = ((Double)values.elementAt(0)).doubleValue();
            if(logScale) lastValue = Math.log(lastValue)/Math.log(10.0); // get log to base 10
            Color lastColor = (Color)colors.elementAt(0);
            String thisLabel = (String)labels.elementAt(0);
            
            drawAxisLabel(g2,x,y, thisLabel);
            
            int xBox = x;
                      
            for(int i=1;i<values.size();i++) {
                double nextValue = ((Double)values.elementAt(i)).doubleValue();
                if(logScale) nextValue=Math.log(nextValue)/Math.log(10.0);
                Color nextColor = (Color)colors.elementAt(i);
                thisLabel = (String)labels.elementAt(i);
 
                double thisFraction = (nextValue-lastValue)/(lastDataPoint-firstDataPoint);
                int xValue = (int)(thisFraction*(double)colorBoxWidth);
//                System.out.println("Drawing for value "+Double.toString(nextValue));
//                System.out.println("Drawing for color "+nextColor.toString());
//                System.out.println("Drawing for label "+thisLabel);
//                System.out.println("Fraction="+Double.toString(thisFraction));
//                System.out.println("xValue "+Integer.toString(xValue));
    
                drawAxisLabel(g2,xBox+xValue,y, thisLabel); 
                
                Color fillColor= lastColor;


                if(!nextColor.equals(lastColor)) {
                    GradientPaint colorGradient = new GradientPaint(xBox, y, lastColor,xValue+xBox,y,nextColor);
                    g2.setPaint(colorGradient);
                }
                else {
                    g2.setPaint(lastColor);
                   }                    

                int xWidth = (i==values.size()-1) ? x+colorBoxWidth-xBox : xValue;
                drawRect = new Rectangle2D.Double(xBox, y, xWidth, colorBoxHeight);
                g2.fill(drawRect);
                // now the tickmarks and labels
                                
                lastColor=nextColor;
                lastValue=nextValue;
                xBox+=xValue;
            }
            
            // now draw the outline
             drawRect = new Rectangle2D.Double(x, y, colorBoxWidth, colorBoxHeight);
//             System.out.println("Bad2: "+Double.toString(drawRect.getX())+","+Double.toString(drawRect.getY())+","+Double.toString(drawRect.getWidth())+","+Double.toString(drawRect.getHeight()));
             g2.setPaint(Color.black);
             g2.draw(drawRect);
            
            
 
            
    }

    public void drawSolidColorScale(java.awt.Graphics2D g2, int x, int y) {

            Rectangle2D.Double drawRect;            
            int nboxes = values.size();
            int boxWidth = (int)((double)colorBoxWidth/(double)nboxes);        
            
            int xBox = 0;
            
            for(int i=0;i<values.size();i++) {
                double nextValue = ((Double)values.elementAt(i)).doubleValue();
                Color nextColor = (Color)colors.elementAt(i);
                String thisLabel = (String)labels.elementAt(i);
                
                drawAxisLabel(g2,x+xBox+boxWidth/2,y, thisLabel);                 
                g2.setPaint(nextColor);

                drawRect = new Rectangle2D.Double(x+xBox, y, boxWidth, colorBoxHeight);
                g2.fill(drawRect);
                // now the tickmarks and labels
                                
                xBox+=boxWidth;
            }
            
            // now draw the outline
             drawRect = new Rectangle2D.Double(x, y, nboxes*boxWidth, colorBoxHeight);
//             System.out.println("Bad2: "+Double.toString(drawRect.getX())+","+Double.toString(drawRect.getY())+","+Double.toString(drawRect.getWidth())+","+Double.toString(drawRect.getHeight()));
             g2.setPaint(Color.black);
             g2.draw(drawRect);
                        
    }

    /** Draw axis label at x,y */
    public void drawAxisLabel(java.awt.Graphics2D g2, int x, int y, String label) {
        g2.setPaint(Color.black);

        g2.draw(new Line2D.Double(x, y, x, y-tickHeight));
        g2.setFont(displayScaleFont);
        g2.setPaint(darkBlueColor);
        FontMetrics metrics = g2.getFontMetrics();
        int stringWidth = metrics.stringWidth(label);
        int stringHeight = metrics.getHeight();  
        g2.drawString(label,x-stringWidth/2,y-tickHeight-tickLabelOffSet);
 
    }

    /** get color for given value */
    public Color getColor(double value) {
      if(!isGradientPaint) {
 
         for(int i=0;i<values.size();i++) {
                double thisValue = ((Double)values.elementAt(i)).doubleValue();
                if(value<=thisValue) return (Color)colors.elementAt(i);
                }
          return (Color)colors.lastElement();
         }
       else {
           if(logScale) value = Math.log(value)/Math.log(10.0);
           Color loColor = (Color)colors.elementAt(0);
           double loValue = (logScale) ? Math.log(((Double)values.elementAt(0)).doubleValue())/Math.log(10.0) : ((Double)values.elementAt(0)).doubleValue();
           if(value<=loValue) return loColor;

           for(int i=1;i<values.size();i++) {

             double hiValue = (logScale) ? Math.log(((Double)values.elementAt(i)).doubleValue())/Math.log(10.0) : ((Double)values.elementAt(i)).doubleValue();
             Color hiColor = (Color)colors.elementAt(i);
//             System.out.println("Data between "+Double.toString(loValue)+" and "+Double.toString(hiValue));
             if(value<=hiValue) {
                    int loRed = loColor.getRed();
                    int hiRed = hiColor.getRed();
                    int loBlue = loColor.getBlue();
                    int hiBlue = hiColor.getBlue();
                    int loGreen = loColor.getGreen();
                    int hiGreen = hiColor.getGreen();
                    double fraction = (value-loValue)/(hiValue-loValue);
//                    System.out.println("fraction="+Double.toString(fraction));
//                    System.out.println("scalecolor: "+loRed+","+hiRed+":"+loGreen+","+hiGreen+":"+loBlue+","+hiBlue);
                    if(fraction<0.0) fraction=0.0;
                    if(fraction>1.0) fraction=1.0;
                    double redValue = (double)(hiRed-loRed)*fraction;
                    double blueValue = (double)(hiBlue-loBlue)*fraction;
                    double greenValue = (double)(hiGreen-loGreen)*fraction;
//                    int newRed=loRed+(int)redValue;
//                    int newGreen=loGreen+(int)greenValue;
//                    int newBlue=loBlue+(int)blueValue;
//                    System.out.println("newcolor: "+newRed+","+newGreen+","+newBlue);
                    return new Color(loRed+(int)redValue,loGreen+(int)greenValue,loBlue+(int)blueValue);
                    }
               loColor=hiColor;
               loValue=hiValue;
              }
            return loColor;  // ie, return the hiColor because value must be > greatesthiValue
           }

    }
// return the label corresponding to the value
// if not defined, then return the value as a String
  public String getLabel(String value) {
         try {
          for(int i=0;i<values.size();i++) {
                double thisValue = ((Double)values.elementAt(i)).doubleValue();
                if(Double.parseDouble(value)==thisValue) {
                           String theLabel = (String)labels.elementAt(i);
                           if(theLabel.indexOf("<")==-1 && theLabel.indexOf(">")==-1) return theLabel;
                           }
                }
          }catch(Exception e){};  // in case value cannot be represented as a number
          return value;
         }
    
}
